@@ -2,10 +2,13 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-// +build amd64 amd64p32 386 arm ppc64le s390x arm64
+//go:build amd64 || 386 || arm || ppc64le || ppc64 || s390x || arm64
+// +build amd64 386 arm ppc64le ppc64 s390x arm64
 
 package md5
 
+const haveAsm = true
+
 //go:noescape
 
 func block(dig *digest, p []byte)
